package tutorial.lookandfeel.radiobuttongroup;

import java.lang.IllegalArgumentException;
import java.io.Serializable;
import java.util.Vector;
import java.util.Enumeration;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Color;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.AWTEvent;
import java.awt.AWTEventMulticaster;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.ItemSelectable;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;
import javax.swing.ButtonModel;
import javax.swing.ButtonGroup;
import javax.swing.border.Border;
import javax.swing.BorderFactory;
import javax.swing.border.TitledBorder;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.AbstractButton;
import javax.swing.JRadioButton;

public class RadioButtonGroupPanel extends JPanel implements ActionListener, ItemSelectable, Serializable {
    
    public final static Color BlueGray = new Color(102, 102, 153);
    public final static Color PaleYellow = new Color(255, 255, 204);
    
    private transient java.awt.event.ItemListener m_listener = null;  // do not serialize listeners
    
    private Object[] m_selected = null;  // required for the ItemSelectable interface
    private ButtonGroup m_group = null;  // implements single-selection for the group of radio buttons
    
    private TitledBorder m_border = null;  // title for the group of buttons

    public RadioButtonGroupPanel() {
        setForeground(PaleYellow);
        setBackground(BlueGray);
 
        Border etched = BorderFactory.createEtchedBorder();
        m_border = BorderFactory.createTitledBorder(etched, "Single Option Selector");  // place-holder title
        m_border.setTitleColor(getForeground());
        setBorder(m_border);

        enableEvents(AWTEvent.ITEM_EVENT_MASK);  // must fire item events
        
        m_group = new ButtonGroup();  // initially, group is empty
        m_selected = new Object[0];  // and nothing is selected
    }
        
    private AbstractButton[] getButtons() {
        Vector vector = new Vector();
        int n = getComponentCount();
        for(int i =0; i < n; i++) {
            JComponent jc = (JComponent)getComponent(i);
            if(jc instanceof AbstractButton) {
                vector.addElement(jc);
            }
        }
        if(vector.isEmpty()) { return null; }

        AbstractButton results[] = new AbstractButton[vector.size()];
        vector.copyInto(results);
        return results;
    }

    public String getGroupLabel() { return m_border.getTitle(); }

    public synchronized void setGroupLabel(String label) {
        m_border.setTitle(label);
        m_border.setTitleColor(getForeground());  // in case the panel's fg color changed
        revalidate();
        repaint();
    }

    public String[] getButtonLabels() {
        AbstractButton buttons[] = getButtons();
        if(buttons == null) { return null; }
        
        String labels[] = new String[buttons.length];
        for(int i =0; i < buttons.length; i++) {
            labels[i] = buttons[i].getText();
        }
        return labels;
    }
    
    public synchronized void setButtonLabels(String labels[]) {
        AbstractButton buttons[] = getButtons();
        if(buttons != null) {
            for(int i =0; i < buttons.length; i++) {
                remove(buttons[i]);
            }
        }

        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints constraints = new GridBagConstraints();
        setLayout(gridbag);
        
        m_group = new ButtonGroup();  // empty group
        m_selected = new Object[0];  // nothing is selected

        constraints.insets = new Insets(9, 15, 9, 15);  // top, left, ...

        for(int i =0, x =0, y =0; i < labels.length; i++, x++) {  // x and y refer to the current gridbag cell
            boolean selected = (i == 0) ? true : false;  // default selection
            JRadioButton rb = new JRadioButton(labels[i], selected);
            rb.setActionCommand(labels[i]);
            rb.setForeground(getForeground());
            rb.setBackground(getBackground());
            rb.getModel().addActionListener(this);
            m_group.add(rb);
            constraints.gridx = x;
            constraints.gridy = y;
            gridbag.setConstraints(rb, constraints);
            add(rb);
        }
        revalidate();
        repaint();
    }

    public String getSelection() { 
        ButtonModel model = m_group.getSelection();
        
        if(model != null) {
            AbstractButton buttons[] = getButtons();
            if(buttons != null) {
                for(int i =0; i < buttons.length; i++) {
                    if(buttons[i].getModel() == model) {
                        return buttons[i].getText();
                    }
                }
            }
        }
        return null;
    }
    
    public synchronized void setSelection(String label) {
        AbstractButton buttons[] = getButtons();
        if(buttons != null) {
            for(int i =0; i < buttons.length; i++) {
                if(buttons[i].getText().equals(label)) {
                    buttons[i].getModel().setSelected(true);
                    return;
                 }
            }
        }
        throw(new IllegalArgumentException("The group does not include a radio button labelled: " + label));
    }
    
    public synchronized void addItemListener(java.awt.event.ItemListener listener) {
        m_listener = AWTEventMulticaster.add(m_listener, listener);
    }
    
    public void removeItemListener(java.awt.event.ItemListener listener) {
        m_listener = AWTEventMulticaster.remove(m_listener, listener);
    }

    public Object[] getSelectedObjects() {
        return m_selected;
    }
    
    public void actionPerformed(ActionEvent ae) {
       if(getSelection().equals(ae.getActionCommand())) {
            m_selected = new Object[1];
            m_selected[0] = ae.getSource();
            ItemEvent ie = new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED, ae.getSource(), ItemEvent.SELECTED);
            if(m_listener != null) {
                m_listener.itemStateChanged(ie);
            }
        }
    }
}
